#include "DistrhoPlugin.hpp"

START_NAMESPACE_DISTRHO

    class TabPlugin : public Plugin
{
public:
    TabPlugin()
        : Plugin(kParameterCount, 0, 0)
    {
        // clear all parameters
        std::memset(fParameters, 0, sizeof(float) * kParameterCount);

    }

protected:
    const char* getLabel() const override       {   return "yaw-tab"; }
    const char* getDescription() const override {   return "Drawing tablet synth UI"; }
    const char* getMaker() const override       {   return "yaw-audio"; }
    const char* getHomePage() const override    {   return "https://yaw.man/plugins/yaw-tab"; }
    const char* getLicense() const override     {   return "ISC"; }
    uint32_t    getVersion() const override     {   return d_version(1, 0, 0); }
    int64_t     getUniqueId() const override    {   return d_cconst('y', 'w', 't', 'b'); }


    void initParameter(uint32_t index, Parameter& parameter) override
    {
        parameter.hints = kParameterIsAutomable | kParameterIsOutput;
        parameter.ranges.def = 0.0f;
        parameter.ranges.min = 0.0f;
        parameter.ranges.max = 16777216.0f;

        switch (index)
        {
        case kParameterTabletX:
            parameter.name = "x";
            parameter.symbol = "x";
            parameter.ranges.min = 0;
            parameter.ranges.max = 1024;
            break;
        case kParameterTabletY:
            parameter.name = "y";
            parameter.symbol = "y";
            parameter.ranges.min = 0;
            parameter.ranges.max = 1024;
            break;
        case kParameterTabletZ:
            parameter.name = "z";
            parameter.symbol = "z";
            parameter.ranges.min = 0;
            parameter.ranges.max = 1024;
            break;
        case kParameterTabletPressure:
            parameter.name = "p";
            parameter.symbol = "p";
            parameter.ranges.min = 0;
            parameter.ranges.max = 1024;
            break;
        }
    }

    float getParameterValue(uint32_t index) const override
    {
        return fParameters[index];

    }

    void setParameterValue(uint32_t, float) override
    {
        // this is only called for input parameters, which we have none of.
    }

    void run(const float** inputs, float** outputs, uint32_t frames) override
    {
        /**
           This plugin does nothing, it just demonstrates information usage.
           So here we directly copy inputs over outputs, leaving the audio untouched.
           We need to be careful in case the host re-uses the same buffer for both inputs and outputs.
         */
        if (outputs[0] != inputs[0])
            std::memcpy(outputs[0], inputs[0], sizeof(float) * frames);

        if (outputs[1] != inputs[1])
            std::memcpy(outputs[1], inputs[1], sizeof(float) * frames);

        // get time position
        const TimePosition& timePos(getTimePosition());
        fParameters[kParameterTabletPressure] = timePos.bbt.tick;
    }

private:
    float fParameters[kParameterCount];

    DISTRHO_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR(TabPlugin)
};

Plugin* createPlugin()
{
    return new TabPlugin();
}

END_NAMESPACE_DISTRHO
