#include <string>
#include <Windows.h>
#include "MSGPACK.H"
#include "wintab.h"

#define PACKETDATA	(PK_X | PK_Y | PK_Z | PK_BUTTONS | PK_NORMAL_PRESSURE)
#define PACKETMODE	PK_BUTTONS
#include "pktdef.h"
#include "wtutil.h"

class Tablet {
public:

	Tablet(HWND hwnd) {
		if (!hwnd) { return; }

		if (!LoadWintab() || !gpWTInfoA(0, 0, NULL)) {
			return;
		}

		wintabAvailable = true;
		NewContext(hwnd);
	}

	~Tablet() {
		gpWTClose(hctx); //Chui pas certain s'il faut fermer le context.
		UnloadWintab();
	}

	void NewPacket() {
		//Serial number of newest packet.
		UINT oldest, newest;
		gpWTQueuePacketsEx(hctx, &oldest, &newest);

		//Store newest packet in pkt, flush older packets.
		gpWTPacket(hctx, newest, &pkt);
	}

	bool wintabAvailable = false;
	PACKET pkt = { 0 };
	UINT maxPressure = 1;

private:

	HCTX hctx = NULL;
	UINT wDevice = 0;
	AXIS TabletX = { 0 };
	AXIS TabletY = { 0 };
	AXIS TabletZ = { 0 };
	AXIS TabletPressure = { 0 };
	LOGCONTEXT ctx = {};

	void NewContext(HWND hwnd) {
		if (hctx) { gpWTClose(hctx); }

		ctx.lcOptions |= CXO_SYSTEM;
		gpWTInfoA(WTI_DEFSYSCTX, 0, &ctx);
		ctx.lcOptions |= CXO_MESSAGES; //TODO: checker ela
		ctx.lcPktData = PACKETDATA;
		ctx.lcPktMode = PACKETMODE;
		
		//Tablet extents.
		gpWTInfoA(WTI_DEVICES, DVC_X, &TabletX);
		gpWTInfoA(WTI_DEVICES, DVC_Y, &TabletY);
		gpWTInfoA(WTI_DEVICES, DVC_Z, &TabletZ);
		gpWTInfoA(WTI_DEVICES, DVC_NPRESSURE, &TabletPressure);

		ctx.lcInOrgX = 0;
		ctx.lcInOrgY = 0;
		ctx.lcInOrgZ = 0;
		ctx.lcInExtX = TabletX.axMax;
		ctx.lcInExtY = TabletY.axMax;
		ctx.lcInExtZ = TabletZ.axMax;
		maxPressure  = TabletPressure.axMax;

		ctx.lcOutOrgX = GetSystemMetrics(SM_XVIRTUALSCREEN);
		ctx.lcOutOrgY = GetSystemMetrics(SM_YVIRTUALSCREEN);
		ctx.lcOutExtX = GetSystemMetrics(SM_CXVIRTUALSCREEN);
		ctx.lcOutExtY = -GetSystemMetrics(SM_CYVIRTUALSCREEN);
		ctx.lcOutOrgZ = 0;
		ctx.lcOutExtZ = ctx.lcInExtZ;

		hctx = gpWTOpenA(hwnd, &ctx, FALSE);

		gpWTEnable(hctx, TRUE);
	}


};